@@ -175,8 +175,8 @@ class Train(Vehicle, metaclass=abc.ABCMeta):
 
     def __init__(self):
         self._initialized = False
-        self._static_data: StaticDataConnector = ...
-        self._dynamic_data: DynamicDataConnector = ...
+        self._static_data: StaticDataConnector
+        self._dynamic_data: DynamicDataConnector
 
     def __enter__(self):
         self.init()
@@ -418,21 +418,31 @@ class _LazyStation(Station):
     """
     The LazyStation is a Station that maybe does not yet have information on connecting trains.
     If it does not have information by the time it is requested by the user,
-    it will then proceed to load the information through ``lazy_func``.
+    it will then proceed to load the information through ``lazy_func(self.id)``.
     """
 
     __slots__ = ["_lazy_func", "_cache_valid_until", "_cache_timeout"]
 
-    def __init__(self, *args, lazy_func: Optional[Callable[..., List[ConnectingTrain]]] = None, **kwargs):
+    def __init__(self, *args, lazy_func: Optional[Callable[..., List[ConnectingTrain]]] = None,
+                 _cache_timeout: int = 60, **kwargs):
         super().__init__(*args, **kwargs)
         self._lazy_func = lazy_func
         self._cache_valid_until = 0
-        self._cache_timeout = 60
+        self._cache_timeout = _cache_timeout
 
     @property
     def connections(self) -> Optional[List[ConnectingTrain]]:
-        def request_data():
-            connections = self._lazy_func()
+        """
+        The connecting services departing from this station
+
+        :return: A list of ConnectingTrain objects
+        :rtype: List[ConnectingTrain]
+        """
+        def request_data() -> List[ConnectingTrain]:
+            """
+            Perform the request to get the data and return the response
+            """
+            connections = self._lazy_func(self.id)
             self._connections = connections
             self._cache_valid_until = time.time() + self._cache_timeout  # Cache this result for the next minute
             return connections
