@@ -4,14 +4,13 @@ This module contains everything that has to do with data and data management
 
 import abc
 import json
+import logging
 import threading
 import time
-from os import PathLike
-
 import requests
 
-from typing import Any, Optional, TypeVar, Generic, ItemsView, Union
-
+from os import PathLike
+from typing import Any, Optional, TypeVar, Generic, ItemsView, Union, Sequence
 from requests import Response, RequestException
 
 from .conversions import coordinates_decimal_to_dms
@@ -231,35 +230,30 @@ class DataConnector(metaclass=abc.ABCMeta):
     A class for retrieving data from an API
     """
 
-    __slots__ = ["base_url", "_session", "_cache", "_verify"]
+    __slots__ = ["base_url", "_session", "_cache"]
 
-    def __init__(self, base_url: str, *, verify: bool = True) -> None:
+    def __init__(self, base_url: str) -> None:
         """
         Initialize a new :class:`DataConnector`
 
         :param base_url: The base url of the server to connect to
         :type base_url: str
-        :param verify: Whether to verify the TLS certificate of the server
-        :type verify: bool
         """
         self.base_url = base_url
         self._session = requests.Session()
         self._cache = DataStorage()
-        self._verify = verify
 
     def __del__(self):
         self._session.close()
 
-    def _get(self, endpoint: str, *args: Any, **kwargs: Any) -> Response:
+    def _get(self, endpoint: str, *args: Any, base_url: str = None, **kwargs: Any) -> Response:
         """
         Request data from the server
 
-        :param endpoint: The endpoint to request data from
-        :type endpoint: str
-        :param args: args to pass to the request
-        :type args: Any
-        :param kwargs: kwargs to pass to the request
-        :type kwargs: Any
+        :param str endpoint: The endpoint to request data from
+        :param Any args: args to pass to the request
+        :param str base_url: An optional different base url to use for the request
+        :param Any kwargs: kwargs to pass to the request
         :return: The response from the server
         :rtype: Response
         """
@@ -271,10 +265,12 @@ class DataConnector(metaclass=abc.ABCMeta):
         kwargs.update({
             "headers": headers,
             "timeout": 1,
-            "verify": kwargs.get("verify", self._verify)
+            "verify": True if kwargs.get("verify") is None else bool(kwargs.get("verify"))
         })
+        # Allow a different base url, but use self.base_url as default
+        base_url = self.base_url if base_url is None else base_url
         try:
-            return self._session.get(f"https://{self.base_url}{endpoint}", *args, **kwargs)
+            return self._session.get(f"https://{base_url}{endpoint}", *args, **kwargs)
         except RequestException as e:
             raise APIConnectionError() from e
 
@@ -301,22 +297,6 @@ class DataConnector(metaclass=abc.ABCMeta):
         """
         self._cache.set(key, value)
 
-    def export(self, path: Union[str, PathLike]) -> None:
-        """
-        Export the cache to a file
-
-        :param path: The path to export to
-        :type path: Union[str, PathLike]
-        :return: Nothing
-        :rtype: None
-        """
-        data = {}
-        for key, value in self._cache.items():
-            data[key] = value
-
-        with open(path, "w") as f:
-            json.dump(data, f)
-
     @abc.abstractmethod
     def refresh(self) -> None:  # pragma: no cover
         """
@@ -335,7 +315,7 @@ class StaticDataConnector(DataConnector, metaclass=abc.ABCMeta):
     __slots__ = []
 
     def __init__(self, base_url: str, *args, **kwargs):
-        super().__init__(base_url, *args, **kwargs)
+        super().__init__(base_url)
 
 
 class DynamicDataConnector(DataConnector, metaclass=abc.ABCMeta):
@@ -343,16 +323,24 @@ class DynamicDataConnector(DataConnector, metaclass=abc.ABCMeta):
     A :class:`DataConnector` for data that changes frequently and therefore has to be requested continuously
     """
 
-    __slots__ = ["_runner", "_running", "_initialized", "optimize"]
+    __slots__ = ["_runner", "_running", "_initialized", "optimize", "silent"]
 
     def __init__(self, base_url: str, *args, **kwargs):
-        super().__init__(base_url, *args, **kwargs)
+        super().__init__(base_url)
         self._runner = threading.Thread(
-            target=self._run, name=f"DynamicDataConnector-Runner for '{self.base_url}'", daemon=True
+            target=self._run, name=f"DynamicDataConnector-Runner for '{self.base_url}'", daemon=True,
         )
         self._running = False
         self._initialized = False
+
         self.optimize = True
+        """
+        Whether to adapt the refresh rate of the data connector to be as close as possible to 1Hz.
+        """
+        self.silent = True
+        """
+        Suppress exceptions that occur in the runner thread
+        """
 
     @property
     def connected(self) -> bool:
@@ -388,6 +376,10 @@ class DynamicDataConnector(DataConnector, metaclass=abc.ABCMeta):
                 self.refresh()
             except APIConnectionError as e:
                 self._running = False
+                if self.silent:
+                    continue  # End loop, terminate thread cleanly
+
+                # If not silent, raise the exception
                 raise e
 
             # Signal that the data has been refreshed at least once (for thread synchronization)
@@ -444,13 +436,38 @@ class JSONDataConnector(DataConnector, metaclass=abc.ABCMeta):
 
     __slots__ = []
 
-    def _get(self, endpoint: str, *args: Any, **kwargs: Any) -> dict:
+    def _get(self, endpoint: str, *args: Any, base_url: str = None, **kwargs: Any) -> dict:
         headers = kwargs.get("headers", {})
         headers.update({
             "accept": "application/json",
         })
         kwargs["headers"] = headers
         try:
-            return super(JSONDataConnector, self)._get(endpoint, *args, **kwargs).json()
+            return super(JSONDataConnector, self)._get(endpoint, *args, base_url=base_url, **kwargs).json()
         except json.JSONDecodeError as e:
+            logging.debug(f"Failed to parse json ({e.__class__.__name__}): {'; '.join(e.args)}")
             raise DataInvalidError() from e
+
+    def export(self, path: Union[str, PathLike], keys: Sequence = None) -> None:
+        """
+        Export the cache to a json file
+
+        :param Union[str, PathLike] path: The path to export to
+        :param Sequence keys: The specific keys to export, if empty export all keys
+        :return: Nothing
+        :rtype: None
+        """
+        data = {}
+        for key, value in self._cache.items():
+            if keys is None:
+                data[key] = value
+            # assume keys is Sequence
+            elif key in keys:
+                data[key] = value
+            # else pass
+
+        if not str(path).endswith(".json"):
+            path = f"{path}.json"
+
+        with open(path, "w") as f:
+            json.dump(data, f, default=lambda o: o.__dict__)
